
/*
    lcd_shift.h
    print and setting functions for a HD44780 compatible display
    Copyright (C) 2011  Marc Heimann <mheimann84@googlemail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LCD_SHIFT_H
#define LCD_SHIFT_H

#include <avr/io.h>


// processor output (shift register & display input) //
// pin configuration
#define PIN_CLOCK		0b10000000
#define PIN_DATA		0b01000000
#define PIN_ENABLE		0b00100000

// port configuration
#define PORT_OUT_CLOCK		PORTD_OUT
#define PORT_OUT_DATA		PORTD_OUT
#define PORT_OUT_ENABLE		PORTD_OUT

#define PORT_DIR_CLOCK		PORTD_DIR
#define PORT_DIR_DATA		PORTD_DIR
#define PORT_DIR_ENABLE		PORTD_DIR

// shift register output (display input) //
#define SHIFT_OUT_RS		0b00001000
#define SHIFT_OUT_DB0_DB4	0b00010000
#define SHIFT_OUT_DB1_DB5	0b00100000
#define SHIFT_OUT_DB2_DB6	0b01000000
#define SHIFT_OUT_DB3_DB7	0b10000000


// display instructions //
// HD44780.pdf, p. 11, figure 4
#define CHARACTERS_PER_LINE	40
#define LINE_1_ADDR		0x00
#define LINE_2_ADDR		0x40

#define PIN_DB0			0b00000001
#define PIN_DB1			0b00000010
#define PIN_DB2			0b00000100
#define PIN_DB3			0b00001000
#define PIN_DB4			0b00010000 
#define PIN_DB5			0b00100000
#define PIN_DB6			0b01000000
#define PIN_DB7			0b10000000

// HD44780.pdf, p. 24, 25
#define CLEAR_DISPLAY			PIN_DB0

#define RETURN_HOME			PIN_DB1

#define ENTRY_MODE_SET			PIN_DB2
#define ENTRY_MODE_SET_ID		PIN_DB1
#define ENTRY_MODE_SET_S		PIN_DB0

#define DISPLAY_ON_OFF_CONTROL		PIN_DB3
#define DISPLAY_ON_OFF_CONTROL_D	PIN_DB2
#define DISPLAY_ON_OFF_CONTROL_C	PIN_DB1
#define DISPLAY_ON_OFF_CONTROL_B	PIN_DB0

#define CURSOR_OR_DISPLAY_SHIFT		PIN_DB4
#define CURSOR_OR_DISPLAY_SHIFT_SC	PIN_DB3
#define CURSOR_OR_DISPLAY_SHIFT_RL	PIN_DB2

#define FUNCTION_SET			PIN_DB5
#define FUNCTION_SET_DL			PIN_DB4
#define FUNCTION_SET_N			PIN_DB3
#define FUNCTION_SET_F			PIN_DB2

#define SET_DDRAM_ADDRESS		PIN_DB7

// composed instructions (not complete, modify or add your own instructions) 
#define SET_4BIT_ONE_LINE		FUNCTION_SET
#define SET_4BIT_TWO_LINES		FUNCTION_SET | FUNCTION_SET_N
#define SET_DISPLAY_ON			DISPLAY_ON_OFF_CONTROL | DISPLAY_ON_OFF_CONTROL_D
#define SET_DISPLAY_ON_WITH_CURSOR	DISPLAY_ON_OFF_CONTROL | DISPLAY_ON_OFF_CONTROL_D | DISPLAY_ON_OFF_CONTROL_C
#define INCREMENT_CURSOR		ENTRY_MODE_SET | ENTRY_MODE_SET_ID
#define SHIFT_LEFT			CURSOR_OR_DISPLAY_SHIFT | CURSOR_OR_DISPLAY_SHIFT_SC
#define SHIFT_RIGHT			CURSOR_OR_DISPLAY_SHIFT | CURSOR_OR_DISPLAY_SHIFT_SC | CURSOR_OR_DISPLAY_SHIFT_RL

// function declarations
extern void lcd_enable( void );
extern void shift_byte( uint8_t data_byte );
extern void send_instr( uint8_t data );
extern void send_instruction( uint8_t data, uint16_t usec_delay );
extern void send_data( uint8_t data );
extern void init_lcd( void );
extern void print_lcd( char* buffer, uint8_t addr );

#endif // LCD_SHIFT_H

